﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Airplayne.Playback;
using Airplayne.Model;
using Airplayne.Interface;
using Airplayne.Model.Repository;
using System.Threading;
using System.Runtime.InteropServices;
using Airplayne.Tools;
using System.Reflection;
using Airplayne.Interface.Hotkeys;
using Airplayne.Model.Tags;

namespace Airplayne
{
  public class Factory
  {
    public Factory() { }

    private Dictionary<string, Player> _players = new Dictionary<string, Player>();
    private SettingsManager _settingsManager;
    private LibraryManager _libraryManager;
    private PlaylistManager _playlistManager;
    private BagListManager _bagListManager;
    private HotkeyManager _hotkeyManager;

    public void Initialize()
    {
      _settingsManager = new SettingsManager(SetPlayer);
      _libraryManager = new LibraryManager(_settingsManager);
      _playlistManager = new PlaylistManager(_settingsManager, _libraryManager);
      _bagListManager = new BagListManager(_settingsManager, _libraryManager);
      _playlistManager.SisterListManager = _bagListManager;
      _bagListManager.SisterListManager = _playlistManager;
      _settingsManager.VolumeChanged += (volume) => _playlistManager.SetPlayerVolume(volume);
      _settingsManager.LibrarySettingsChanged += _libraryManager.ScanInBackground;
      _hotkeyManager = new HotkeyManager(_settingsManager, _playlistManager);
      _playlistManager.LoadState();
      _bagListManager.LoadState();

      ThreadPool.QueueUserWorkItem((state) =>
      {
        LoadPlayer();
        _libraryManager.LoadLibraryFromDisk();
      });
    }

    private void LoadPlayer()
    {
      foreach (SettingsManager.PlayerDescription playerDescription in
        _settingsManager.AvailablePlayersByPriority)
      {
        try
        {
          _settingsManager.PlayerType = playerDescription;
          break;
        }
        catch (PlayerInitializationException)
        {
          continue;
        }
      }
    }

    private Player GetPlayer(SettingsManager.PlayerDescription playerDescription)
    {
      if (!_players.ContainsKey(playerDescription.Name))
      {
        try
        {
          _players[playerDescription.Name] = (Player)Activator.CreateInstance(playerDescription.ImplementingType);
        }
        catch (TargetInvocationException ex)
        {
          throw ex.InnerException; // a no-no, but I really don't need the trace in this case.
        }
      }

      return _players[playerDescription.Name];
    }

    private void SetPlayer(SettingsManager.PlayerDescription playerDescription)
    {
      try
      {
        Player player = GetPlayer(playerDescription);
        player.SetVolume(_settingsManager.Volume);

        _playlistManager.SetPlayer(player);
      }
      catch (PlayerInitializationException ex)
      {
        Logger.Write(ex, "Failed creating player '{0}'", playerDescription.Name);
        throw;
      }
    }

    public MainForm GetUI()
    {
      return new MainForm(_playlistManager, _bagListManager, _settingsManager, _libraryManager);
    }

  }
}
