﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Airplayne.Interface.Controls
{
  [ToolStripItemDesignerAvailability(ToolStripItemDesignerAvailability.All)]
  public class BarLabel : ToolStripLabel
  {
    private Rectangle frameRect;
    private bool empty;
    private int _value;

    public BarLabel()
      : base()
    {
      Style = Styles.Plain;
    }

    #region Event
    public delegate void ValueSelectedHandler(int value);
    public event ValueSelectedHandler ValueSelected;
    #endregion

    #region Properties
    public bool SetValueOnDrag { get; set; }

    public int Total { get; set; }

    public int Value
    {
      get { return _value; }
      set
      {
        _value = value;
        Invalidate();
      }
    }

    public bool Empty
    {
      get { return empty; }
      set
      {
        empty = value;
        if (empty)
          Text = ToolTipText = string.Empty;
        Invalidate();
      }

    }

    public enum Styles { Plain, Triangle };

    public Styles Style { get; set; }

    #endregion

    protected override void OnLayout(LayoutEventArgs e)
    {
      base.OnLayout(e);
      frameRect = new Rectangle(4, this.Height - 4, this.Width - 9, 3);
    }

    protected override void OnPaint(PaintEventArgs e)
    {
      if (!Empty && Total > 0)
      {
        Brush fillBrush = new SolidBrush(SystemColors.HotTrack);
        Pen pen = new Pen(SystemColors.ControlDarkDark);
        int pos = Value * frameRect.Width / Total;

        if (Style == Styles.Plain)
          PaintPlainBar(e, fillBrush, pen, pos);
        else
          PaintTriangleBar(e, fillBrush, pen, pos);
      }
      base.OnPaint(e);
    }

    private void PaintTriangleBar(PaintEventArgs e, Brush fillBrush, Pen pen, int pos)
    {
      e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

      e.Graphics.DrawPolygon(pen, new Point[]{
            new Point(frameRect.Left, frameRect.Bottom),
            new Point(frameRect.Right, frameRect.Top - 3),
            new Point(frameRect.Right, frameRect.Bottom)
          });
      e.Graphics.FillPolygon(fillBrush, new Point[] {
            new Point(frameRect.Left + 1, frameRect.Bottom),
            new Point(frameRect.Left + pos, frameRect.Bottom),
            new Point(frameRect.Left + pos, frameRect.Bottom - (frameRect.Height+3)*Value/Total)
          });
    }

    private void PaintPlainBar(PaintEventArgs e, Brush fillBrush, Pen pen, int pos)
    {
      e.Graphics.SmoothingMode = SmoothingMode.Default;

      e.Graphics.DrawRectangle(pen, frameRect);
      e.Graphics.FillRectangle(
        fillBrush, frameRect.Left + 1,
        frameRect.Top + 1, pos - 1, frameRect.Height - 1);
    }

    protected override void OnMouseMove(MouseEventArgs e)
    {
      if (e.Button == MouseButtons.Left && SetValueOnDrag)
        SetValue(e);

      base.OnMouseMove(e);
    }

    private void SetValue(MouseEventArgs e)
    {
      if (frameRect.Width > 0)
      {
        int value = (e.X - frameRect.Left) * Total / frameRect.Width;

        var handler = ValueSelected;
        if (handler != null)
          handler(value);
      }
    }

    protected override void OnMouseDown(MouseEventArgs e)
    {
      SetValue(e);

      base.OnMouseDown(e);
    }
  }
}