using System;
using System.Drawing;
using System.Windows.Forms;
using Airplayne.Model.Music;
using Airplayne.Tools.Threading;
using Airplayne.Tools;
using System.Threading;
using Airplayne.Interface.Util;

namespace Airplayne.Interface.Controls
{
  public partial class CoverDisplay : UserControl
  {
    private Album _lastRequestedAlbum;
    private object _coverLock = new object();
    private bool _hidden = false;
    private RequestQueue<Album, Image> _thumbnailRequester;
    private int? _maxCoverSize;

    public CoverDisplay()
    {
      InitializeComponent();
      _thumbnailRequester = new RequestQueue<Album, Image>(GetCoverThumbnail, 50, ThreadPriority.Lowest);
      _thumbnailRequester.RequestFailed += _thumbnailRequester_RequestFailed;
      _thumbnailRequester.ResponseReady += _thumbnailRequester_ResponseReady;
    }

    private int MaxCoverSize
    {
      get
      {
        if (!_maxCoverSize.HasValue)
          _maxCoverSize = Math.Min(Width, Height);
        return _maxCoverSize.Value;
      }
    }

    private void _thumbnailRequester_ResponseReady(Album requestParameter, Image response)
    {
      if (object.Equals(_lastRequestedAlbum, requestParameter))
      {
        if (response != null)
        {
          ShowCover(response);
        }
        else
        {
          ClearCover();
        }
      }
    }

    private void ShowCover(Image response)
    {
      this.TryBeginInvoke((Action)delegate
      {
        SetImageAndFit(response);
        Visible = !_hidden;
      });
    }

    private void SetImageAndFit(Image thumb)
    {
      int right = Right;
      int bottom = Bottom;

      BackgroundImage = thumb;
      Width = thumb.Width;
      Height = thumb.Height;

      var location = Location;
      location.Offset(right - Right, bottom - Bottom);
      Location = location;
    }

    private void _thumbnailRequester_RequestFailed(Album request, Exception exception)
    {
      Logger.Write(exception, "Exception loading thumbnail for '{0}'", request);
      ClearCover();
    }

    public void LoadCoverFor(Album album)
    {
      if (!object.Equals(_lastRequestedAlbum, album))
        ClearCover();

      _lastRequestedAlbum = album;
      _thumbnailRequester.Request(album);
    }

    private Image GetCoverThumbnail(Album album)
    {
      return CoverHelper.GetCover(album, MaxCoverSize);
    }

    private void ClearCover()
    {
      this.TryBeginInvoke((Action)delegate()
      {
        this.BackgroundImage = null;
        this.Visible = this.DesignMode;
      });
      this._lastRequestedAlbum = null;
    }

    public void HideCover()
    {
      this.Visible = false;
      _hidden = true;
    }

    public void ShowCover()
    {
      if (this.BackgroundImage != null)
        this.Visible = true;
      _hidden = false;
    }

  }
}
