﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Airplayne.Interface.Util;
using Airplayne.Interface.Hotkeys;

namespace Airplayne.Interface.Controls
{
  public partial class HotkeySelector : UserControl
  {
    public HotkeySelector()
    {
      InitializeComponent();
    }

    protected override void OnLoad(EventArgs e)
    {
      keyDrop.Items.AddRange(ValidKeys.Cast<object>().ToArray());
      SelectedHotkey = null;
      base.OnLoad(e);
    }

    private IEnumerable<UIEnum<Keys>> ValidKeys
    {
      get
      {
        return Enum.GetValues(typeof(Keys)).Cast<Keys>()
          .Where(key => IsValidKey(key))
          .Distinct()
          .Select(key => new UIEnum<Keys>(key)
          {
            NameOverride = Hotkey.GetKeyName(key)
          });
      }
    }

    private bool IsValidKey(Keys key)
    {
      return 
        (key == Keys.None
        || key == Keys.Back
        || key == Keys.Tab
        || key == Keys.Enter
        || key == Keys.Pause
        || key == Keys.Escape
        || (key >= Keys.Space && key <= Keys.Down)
        || key == Keys.PrintScreen
        || (key >= Keys.Insert && key <= Keys.Delete)
        || (key >= Keys.D0 && key <= Keys.Z)
        || (key >= Keys.NumPad0 && key <= Keys.F12)
        || (key >= Keys.BrowserBack && key <= Keys.LaunchApplication2));
    }

    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public Hotkey SelectedHotkey
    {
      get
      {
        var modifier = GetModifierFromCheckBoxes();
        var key = ((UIEnum<Keys>)keyDrop.SelectedItem).Value;

        if (key == Keys.None)
          return null;
        else
          return new Hotkey(modifier, key);
      }
      set
      {
        if (value == null)
          value = new Hotkey(Hotkey.Modifiers.None, Keys.None);

        CheckBoxesFromModifier(value);
        keyDrop.SelectedItem = new UIEnum<Keys>(value.Key);
      }
    }

    private Hotkey.Modifiers GetModifierFromCheckBoxes()
    {
      Hotkey.Modifiers modifier = Hotkey.Modifiers.None;

      if (winCheck.Checked)
        modifier |= Hotkey.Modifiers.Win;
      if (ctrlCheck.Checked)
        modifier |= Hotkey.Modifiers.Control;
      if (altCheck.Checked)
        modifier |= Hotkey.Modifiers.Alt;
      if (shiftCheck.Checked)
        modifier |= Hotkey.Modifiers.Shift;

      return modifier;
    }

    private void CheckBoxesFromModifier(Hotkey hotkey)
    {
      winCheck.Checked = hotkey.HasModifier(Hotkey.Modifiers.Win);
      ctrlCheck.Checked = hotkey.HasModifier(Hotkey.Modifiers.Control);
      altCheck.Checked = hotkey.HasModifier(Hotkey.Modifiers.Alt);
      shiftCheck.Checked = hotkey.HasModifier(Hotkey.Modifiers.Shift);
    }
  }
}
