﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Airplayne.Interface.Controls
{
  /// <summary>
  /// Ignores Up & Down arrows (as this navigates the tree)
  /// 
  /// Properly selects text on focus, as seen on 
  /// http://stackoverflow.com/questions/97459/automatically-select-all-text-on-focus-in-winforms-textbox
  /// </summary>
  public class TextBoxEx : TextBox
  {
    #region Selection on focus
    private bool _alreadyFocused;

    protected override void OnGotFocus(EventArgs e)
    {
      base.OnGotFocus(e);

      // Select all text only if the mouse isn't down.
      // This makes tabbing to the textbox give focus.
      if (MouseButtons == MouseButtons.None)
      {
        SelectAll();
        _alreadyFocused = true;
      } 
    }

    protected override void OnMouseMove(MouseEventArgs e)
    {
      if (MouseButtons != System.Windows.Forms.MouseButtons.None && SelectionLength > 0)
        _alreadyFocused = true;

      base.OnMouseMove(e);
    }

    protected override void OnMouseUp(MouseEventArgs mevent)
    {
      base.OnMouseUp(mevent);

      // Web browsers like Google Chrome select the text on mouse up.
      // They only do it if the textbox isn't already focused,
      // and if the user hasn't selected all text.
      if (!_alreadyFocused && SelectionLength == 0)
      {
        _alreadyFocused = true;
        SelectAll();
      }
    }

    protected override void OnLeave(EventArgs e)
    {
      base.OnLeave(e);

      _alreadyFocused = false;
    }
    #endregion


    protected override void OnKeyDown(KeyEventArgs e)
    {
      if (e.KeyCode == Keys.Up || e.KeyCode == Keys.Down)
        e.Handled = true;

      base.OnKeyDown(e);
    }
  }
}


