﻿using System;
using System.Deployment.Application;
using System.IO;
using System.Windows.Forms;
using Airplayne.Tools;
using Airplayne.Interface.Util;
using System.Linq;

namespace Airplayne.Interface.Dialogs
{
  public partial class HelpDialog : Form
  {
    private static HelpDialog showingDialog;

    public HelpDialog()
    {
      UIUtils.SetFont(this);
      InitializeComponent();
    }

    internal static void ShowHelp(string topicKey = null)
    {
      if (showingDialog != null)
      {
        showingDialog.Activate();
      }
      else
      {
        showingDialog = new HelpDialog();
        showingDialog.Show();
        showingDialog.HandleDestroyed += showingDialog_HandleDestroyed;
      }
      if (topicKey != null)
        showingDialog.ShowTopic(topicKey);
    }

    private void ShowTopic(string topicKey)
    {
      var item = contents.Items.Cast<ListViewItem>()
        .Where(i => (string)i.Tag == topicKey)
        .FirstOrDefault();
      if (item != null)
        item.Selected = true;
    }

    static void showingDialog_HandleDestroyed(object sender, EventArgs e)
    {
      showingDialog.Dispose();
      showingDialog = null;
    }

    private void NavigateTo(string topic)
    {
      string appfolder = Path.GetDirectoryName(Application.ExecutablePath);
      webBrowser.Navigate(appfolder + "/Helpfiles/" + topic + ".html");
    }

    private void contents_ItemActivate(object sender, EventArgs e)
    {
      if (contents.SelectedItems.Count > 0)
        NavigateTo((string)contents.SelectedItems[0].Tag);
    }

    private void HelpDialog_Load(object sender, EventArgs e)
    {
      NavigateTo("index");
    }

    private void webBrowser_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
    {
      if (ApplicationDeployment.IsNetworkDeployed)
        webBrowser.Document.InvokeScript("setVersion",
          new object[] { 
            DeployUtils.Version
          });
    }



  }
}
