﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Airplayne.Interface.Hotkeys;
using Airplayne.Interface.Util;

namespace Airplayne.Interface.Dialogs
{
  public partial class HotkeyAssignmentDialog : Form
  {
    public HotkeyAssignmentDialog()
    {
      UIUtils.SetFont(this);
      InitializeComponent();
    }

    protected override void OnLoad(EventArgs e)
    {
      base.OnLoad(e);
      actionDrop.Items.AddRange(HotkeyAssignment.UIHotkeyActions.ToArray());
      actionDrop.SelectedItem = new UIEnum<HotkeyAssignment.HotkeyActions>(HotkeyAssignment.HotkeyActions.None);
    }

    public HotkeyAssignment HotkeyAssignment
    {
      get
      {
        var hotkey = SelectedHotkey;
        var action = SelectedAction;

        if (hotkey == null || action == HotkeyAssignment.HotkeyActions.None)
          return null;
        else
          return new HotkeyAssignment(hotkey, action);
      }
    }

    private Hotkey SelectedHotkey
    {
      get
      {
        return hotkeySelector.SelectedHotkey;
      }
    }

    private HotkeyAssignment.HotkeyActions SelectedAction
    {
      get
      {
        return ((UIEnum<HotkeyAssignment.HotkeyActions>)actionDrop.SelectedItem).Value;
      }
    }

    public static HotkeyAssignment SelectHotkeyAssignment()
    {
      using (HotkeyAssignmentDialog dialog = new HotkeyAssignmentDialog())
      {
        if (dialog.ShowDialog() == DialogResult.OK)
          return dialog.HotkeyAssignment;
        else
          return null;
      }
    }

    protected override void OnClosing(CancelEventArgs e)
    {
      if (DialogResult == DialogResult.OK)
      {
        if (!ValidateForm())
          e.Cancel = true;
      }

      base.OnClosing(e);
    }

    private bool ValidateForm()
    {
      if (SelectedAction == HotkeyAssignment.HotkeyActions.None)
      {
        MessageBox.Show(this, "No action selected", "Add Hotkey", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return false;
      }
      else if (SelectedHotkey == null)
      {
        MessageBox.Show(this, "No hotkey selected", "Add Hotkey", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return false;
      }
      return true;
    }
  }
}
