﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Airplayne.Interface.Util;
using Airplayne.Model.Music;

namespace Airplayne.Interface.Dialogs
{
  public partial class PlayerErrorDialog : Form
  {
    private Timer _timer;
    private int _countdown = 5;
    private bool _canSkip;

    public PlayerErrorDialog(Track failingTrack, bool canSkip)
    {
      UIUtils.SetFont(this);
      InitializeComponent();
      _canSkip = canSkip;
      trackLabel.Text = string.Format("{0} [{1}]", failingTrack.Title, failingTrack.AlbumInstance.Title);
      headerLabel.Font = new Font(headerLabel.Font, FontStyle.Bold);
      InitButtons();
    }

    private void InitButtons()
    {
      if (_canSkip)
        UpdateButton();
      else
      {
        skipButton.Visible = false;
        AcceptButton = stopButton;
      }
    }

    protected override void OnShown(EventArgs e)
    {
      base.OnShown(e);

      if (_canSkip)
        StartTimer();
    }

    private void StartTimer()
    {
      _timer = new Timer()
      {
        Interval = (int)TimeSpan.FromSeconds(1).TotalMilliseconds
      };
      _timer.Tick += _timer_Tick;
      _timer.Start();
    }

    private void UpdateButton()
    {
      skipButton.Text = string.Format("S&kip ({0})", _countdown);
    }

    void _timer_Tick(object sender, EventArgs e)
    {
      _countdown--;
      UpdateButton();
      if (_countdown == 0)
      {
        DialogResult = DialogResult.OK;
        Close();
      }
    }

    protected override void OnClosing(CancelEventArgs e)
    {
      if (_canSkip)
      {
        _timer.Stop();
        _timer.Dispose();
      }
      base.OnClosing(e);
    }

  }
}
