﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Airplayne.Tools;
using Airplayne.Interface.Util;

namespace Airplayne.Interface.Dialogs
{
  public partial class PlaylistSearchToolWin : Form
  {
    public delegate List<int?> PlaylistSearchHandler(string search);
    public event PlaylistSearchHandler PlaylistSearched;
    private bool _isBagSearch;

    public PlaylistSearchToolWin()
    {
      UIUtils.SetFont(this);
      InitializeComponent();
    }

    void searchBox_KeyUp(object sender, KeyEventArgs e)
    {
      switch (e.KeyCode) {
        case Keys.Up:
        case Keys.Down:
          if (CanBagSearch)
            IsBagSearch = !IsBagSearch;
          break;
      }
    }

    public bool CanBagSearch { get; set; }

    public bool IsBagSearch
    {
      get { return _isBagSearch; }
      set
      {
        _isBagSearch = value;
        playlistLabel.Font = new Font(playlistLabel.Font, !IsBagSearch ? FontStyle.Bold : FontStyle.Regular);
        bagListLabel.Font = new Font(bagListLabel.Font, IsBagSearch ? FontStyle.Bold : FontStyle.Regular);
      }
    }

    public void searchBox_TextChanged(object sender, EventArgs e)
    {
      if (Visible && PlaylistSearched != null)
      {
        List<int?> counts = PlaylistSearched(searchBox.Text);
        UpdateLabel(playlistLabel, counts[0], "playlist");
        UpdateLabel(bagListLabel, counts[1], "bag");
      }
    }

    private void UpdateLabel(Label label, int? count, string listName)
    {
      switch (count)
      {
        case null:
          label.Text = string.Empty;
          break;
        case 1:
          label.Text = string.Format("1 match in {0}", listName);
          break;
        default:
          label.Text = string.Format("{0} matches in {1}", count, listName);
          break;
      }
    }

    protected override bool ProcessDialogKey(Keys keyData)
    {
      switch (keyData)
      {
        case Keys.Enter:
        case Keys.Escape:
          Close();
          return true;
      }
      return false;
    }

    protected override void OnClosing(CancelEventArgs e)
    {
      e.Cancel = true;
      Hide();
      base.OnClosing(e);
    }



  }
}
