﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Airplayne.Interface.Util;

namespace Airplayne.Interface.Hotkeys
{
  public class Hotkey
  {
    /// <summary>
    /// The enumeration of possible modifiers.
    /// </summary>
    [Flags]
    public enum Modifiers : uint
    {
      None = 0,
      Alt = 1,
      Control = 2,
      Shift = 4,
      Win = 8
    }

    public Hotkey()
    {
      Modifier = Modifiers.None;
      Key = Keys.None;
    }

    public Hotkey(Modifiers modifier, Keys key)
    {
      Modifier = modifier;
      Key = key;
    }

    public bool HasModifier(Modifiers modifier)
    {
      return (Modifier & modifier) > 0;
    }

    public Modifiers Modifier { get; set; }
    public Keys Key { get; set; }

    public override string ToString()
    {
      var parts = new List<string>();
      if (HasModifier(Modifiers.Win))
        parts.Add("Win");
      if (HasModifier(Modifiers.Control))
        parts.Add("Ctrl");
      if (HasModifier(Modifiers.Alt))
        parts.Add("Alt");
      if (HasModifier(Modifiers.Shift))
        parts.Add("Shift");

      parts.Add(GetKeyName(Key));

      return string.Join(" + ", parts);
    }

    public override int GetHashCode()
    {
      return this.ToString().GetHashCode();
    }

    public override bool Equals(object other)
    {
      var otherHotkey = other as Hotkey;
      if (otherHotkey == null)
        return false;

      return otherHotkey.Modifier == Modifier
        && otherHotkey.Key == Key;
    }

    public static string GetKeyName(Keys key)
    {
      switch (key)
      {
        case Keys.Back:
          return "Backspace";
        case Keys.PageDown:
          return "Page down";
        case Keys.PageUp:
          return "Page up";
        case Keys.Enter:
          return "Enter";
        case Keys.PrintScreen:
          return "Print screen";
        default:
          if (key >= Keys.D0 && key <= Keys.D9)
            return key.ToString().Substring(1);
          else
            return key.Description();
      }
    }

    
  }
}
