﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel;
using Airplayne.Interface.Util;

namespace Airplayne.Interface.Hotkeys
{
  public class HotkeyAssignment
  {
    public enum HotkeyActions
    {
      None,
      Previous,
      Play,
      [Description("Play/pause")]
      PlayPause,
      Stop,
      Next,
      [Description("Increase volume")]
      VolumeIncrease,
      [Description("Decrease volume")]
      VolumeDecrease,
      [Description("Stop after current")]
      StopAfterCurrent,
      [Description("Autopilot: Disabled")]
      AutopilotDisable,
      [Description("Autopilot: Add tracks")]
      AutopilotTracks,
      [Description("Autopilot: Add albums")]
      AutopilotAlbums
    }

    public static IEnumerable<UIEnum<HotkeyActions>> UIHotkeyActions
    {
      get
      {
        foreach (var value in Enum.GetValues(typeof(HotkeyActions)))
          yield return new UIEnum<HotkeyActions>((HotkeyActions)value);
      }
    }

    public Hotkey Hotkey { get; set; }
    public HotkeyActions HotkeyAction { get; set; }

    public HotkeyAssignment(Hotkey hotkey, HotkeyActions action)
    {
      Hotkey = hotkey;
      HotkeyAction = action;
    }

    public static HotkeyAssignment DeSerialize(string str)
    {
      string[] parts = str.Split('-');
      Hotkey hotkey = new Hotkey(
        (Hotkey.Modifiers)Enum.ToObject(typeof(Hotkey.Modifiers), int.Parse(parts[0])),
        (Keys)Enum.ToObject(typeof(Keys), int.Parse(parts[1])));
      var action = (HotkeyActions)Enum.ToObject(typeof(HotkeyActions), int.Parse(parts[2]));
      return new HotkeyAssignment(hotkey, action);
    }

    public string Serialize()
    {
      return string.Format("{0}-{1}-{2}", (int)Hotkey.Modifier, (int)Hotkey.Key, (int)HotkeyAction);
    }

  }
}
