﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Airplayne.Model;
using Airplayne.Playback;
using Airplayne.Model.Music;

namespace Airplayne.Interface.Hotkeys
{
  public class HotkeyManager
  {
    private SettingsManager _settingsManager;
    private PlaylistManager _playlistManager;
    private KeyboardHook _keyboardHook;
    private List<HotkeyAssignment> _assignments = null;

    public HotkeyManager(SettingsManager settingsManager, PlaylistManager playlistManager)
    {
      _settingsManager = settingsManager;
      _playlistManager = playlistManager;
      _keyboardHook = new KeyboardHook();
      _settingsManager.HotkeySettingsChanged += () => LoadHotkeys();
      _keyboardHook.KeyPressed += _keyboardHook_KeyPressed;

      LoadHotkeys();
    }

    void _keyboardHook_KeyPressed(Hotkey hotkey)
    {
      var action = _assignments
        .Where(assignment => assignment.Hotkey.Equals(hotkey))
        .First()
        .HotkeyAction;
      ExecuteAction(action);
    }

    private void UnregisterHotkeys()
    {
      _keyboardHook.UnregisterAllHotkeys();
    }

    private void ExecuteAction(HotkeyAssignment.HotkeyActions action)
    {
      switch(action) {
        case HotkeyAssignment.HotkeyActions.Previous:
          _playlistManager.Prev();
          break;
        case HotkeyAssignment.HotkeyActions.Play:
          _playlistManager.Play();
          break;
        case HotkeyAssignment.HotkeyActions.PlayPause:
          _playlistManager.PlayPause();
          break;
        case HotkeyAssignment.HotkeyActions.Stop:
          _playlistManager.Stop();
          break;
        case HotkeyAssignment.HotkeyActions.Next:
          _playlistManager.Next();
          break;
        case HotkeyAssignment.HotkeyActions.VolumeDecrease:
          _settingsManager.Volume--;
          break;
        case HotkeyAssignment.HotkeyActions.VolumeIncrease:
          _settingsManager.Volume++;
          break;
        case HotkeyAssignment.HotkeyActions.StopAfterCurrent:
          _playlistManager.StopAfterCurrent = !_playlistManager.StopAfterCurrent;
          break;
        case HotkeyAssignment.HotkeyActions.AutopilotDisable:
          _playlistManager.AutopilotSetting = Playlist.AutopilotMode.Disabled;
          break;
        case HotkeyAssignment.HotkeyActions.AutopilotTracks:
          _playlistManager.AutopilotSetting = Playlist.AutopilotMode.AddTracks;
          break;
        case HotkeyAssignment.HotkeyActions.AutopilotAlbums:
          _playlistManager.AutopilotSetting = Playlist.AutopilotMode.AddAlbums;
          break;
      }
    }

    private void LoadHotkeys() 
    {
      _assignments = _settingsManager.Hotkeys;
      UnregisterHotkeys();
      RegisterHotkeys();
    }

    private void RegisterHotkeys()
    {
      foreach (var assignment in _assignments)
        _keyboardHook.RegisterHotKey(assignment.Hotkey);
    }

  }
}
