﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Airplayne.Interface.Util
{
  public static class EnumUtils
  {
    public static string Description(this Enum value)
    {
      FieldInfo fi = value.GetType().GetField(value.ToString());
      var attribute = (DescriptionAttribute)fi.GetCustomAttributes(typeof(DescriptionAttribute), false).FirstOrDefault();
      return (attribute == null ? CamelCaseToSentence(value.ToString()) : attribute.Description);
    }

    private static string CamelCaseToSentence(string input)
    {
      return Regex.Replace(input, @"\p{Ll}[\p{Lu}\d]", m => m.Value[0] + " " + char.ToLower(m.Value[1]));
    }

  }

  public class UIEnum<TEnum>
  {
    public string NameOverride { get; set; }

    public UIEnum(TEnum value)
    {
      Value = value;
    }
    public TEnum Value { get; set; }
    private Enum EnumValue
    {
      get
      {
        return (Enum)(object)Value; // sigh
      }
    }

    public override string ToString()
    {
      if (!string.IsNullOrEmpty(NameOverride))
        return NameOverride;
      else
        return EnumValue.Description();
    }
    public override bool Equals(object obj)
    {
      UIEnum<TEnum> other = obj as UIEnum<TEnum>;
      if (other == null)
        return false;
      return object.Equals(other.EnumValue, EnumValue);
    }
    public override int GetHashCode()
    {
      return (int)Convert.ChangeType(EnumValue, typeof(int));
    }
  }
}
