﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Airplayne.Model.Music;
using Airplayne.Model.Repository;
using Airplayne.Model;
using Airplayne.Model.Serialization;
using Airplayne.Playback;

namespace Airplayne.Interface.Util
{
  public class OpenSaveHelper
  {
    private SettingsManager _settings;

    public OpenSaveHelper(SettingsManager settings)
    {
      _settings = settings;
    }

    private string MusicAndPlaylistFilter
    {
      get
      {
        return "Music and Playlists|" + string.Join(";",
                                                    _settings.Extensions
                                                    .Union(new List<string> { "m3u" })
                                                    .Select(extension => string.Format("*.{0}", extension))
                                                    .ToArray());
      }
    }

    private string PlaylistFilter
    {
      get
      {
        return "Playlist|*.m3u";
      }
    }


    public void OpenPlaylist(PlaylistManager playlistManager)
    {
      using (OpenFileDialog openDialog = new OpenFileDialog())
      {
        openDialog.Filter = PlaylistFilter;
        openDialog.Title = string.Format("Open {0}", playlistManager.ListType.Description());
        openDialog.Multiselect = false;
        if (openDialog.ShowDialog() == DialogResult.OK)
        {
          playlistManager.TrackOpener.OpenPlaylist(openDialog.FileName);
        }
      }
    }

    public void AddTracks(PlaylistManager playlistManager)
    {
      using (OpenFileDialog openDialog = new OpenFileDialog())
      {
        openDialog.Filter = MusicAndPlaylistFilter;
        openDialog.Title = string.Format("Add Files to {0}", playlistManager.ListType.Description());
        openDialog.Multiselect = true;
        if (openDialog.ShowDialog() == DialogResult.OK)
        {
          playlistManager.TrackOpener.AddTracks(openDialog.FileNames);
        }
      }
    }

    public void SavePlaylist(PlaylistManager playlistManager)
    {
      using (SaveFileDialog saveDialog = new SaveFileDialog())
      {
        saveDialog.AddExtension = true;
        saveDialog.DefaultExt = "m3u";
        saveDialog.Title = string.Format("Save {0}", playlistManager.ListType.Description());
        saveDialog.Filter = PlaylistFilter;
        if (saveDialog.ShowDialog() == DialogResult.OK)
        {
          string path = saveDialog.FileName;
          playlistManager.TrackOpener.SavePlaylist(playlistManager.GetPlaylist(), path);
        }
      }
    }

  }
}
