﻿using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using Airplayne.Model.Music;
using System;
using System.Reflection;
using System.ComponentModel;
using System.Linq;
using System.Text;
using Airplayne.Tools;

namespace Airplayne.Interface.Util
{
  public static class UIUtils
  {
    private const string searchString = @"http://www.google.com/search?q={0}";

    public static void SetFont(Form form)
    {
      form.Font = SystemFonts.MessageBoxFont;
      form.AutoScaleMode = AutoScaleMode.Dpi;
    }

    public static bool HasFocus(Control control)
    {
      if (control.Focused)
        return true;

      foreach (Control c in control.Controls)
        if (HasFocus(c))
          return true;

      return false;
    }

    public static void ShowInExplorer(string path, bool select)
    {
      if (!select)
        Process.Start(path);
      else
        Process.Start("explorer.exe", string.Format("/select, \"{0}\"", path));
    }

    public static void GoogleAlbum(IMusic music)
    {
      if (music == null)
        return;

      System.Diagnostics.Process.Start(string.Format(searchString,
          Uri.EscapeDataString(music.AlbumInstance.Title)));
    }

    public static void GoogleTrack(Track track)
    {
      if (track == null)
        return;

      System.Diagnostics.Process.Start(string.Format(searchString,
        Uri.EscapeDataString(track.Title)));
    }

    public static bool TryBeginInvoke(this Control control, Action action)
    {
      return TryInvoke(control, action, true);
    }

    public static bool TryInvoke(this Control control, Action action)
    {
      return TryInvoke(control, action, false);
    }

    private static bool TryInvoke(this Control control, Action action, bool beginInvoke)
    {
      if (!control.IsHandleCreated)
        return false;

      if (!control.InvokeRequired)
        action();
      else
      {
        try
        {
          if (beginInvoke)
            control.BeginInvoke(action);
          else
            control.Invoke(action);
        }
        catch (Exception ex)
        {
          Logger.Write(ex, "Exception trying to invoke");
          if (ex is InvalidOperationException || ex is ObjectDisposedException)
            return false;
          else
            throw;
        }
      }
      return true;
    }

  }
}
