using System.Windows.Forms;
using Airplayne.Playback;
using Airplayne.Tools;
using Airplayne.Model.Music;
using Airplayne.Model.Repository;
using Airplayne.Model;
using System.ComponentModel;
using Airplayne.Interface.Util;
using Airplayne.Interface.Dialogs;
using System.Collections.Generic;

namespace Airplayne.Interface.Views
{
  public partial class MusicView : UserControl
  {
    private LibraryManager _libraryManager;
    private PlaylistManager _playlistManager;
    private BagListManager _bagListManager;
    private SettingsManager _settingsManager;

    public MusicView()
    {
      InitializeComponent();
      playlistBagSplit.Panel2Collapsed = true;
      playlistControl.AlbumSelected += playlistControl_AlbumSelected;
      bagListControl.AlbumSelected += playlistControl_AlbumSelected;
    }

    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public LibraryManager LibraryManager
    {
      get { return _libraryManager; }
      set
      {
        _libraryManager = value;
        libraryControl.LibraryManager = value;
      }
    }

    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public PlaylistManager PlaylistManager
    {
      get { return _playlistManager; }
      set
      {
        _playlistManager = value;
        libraryControl.PlaylistManager = value;
        playlistControl.PlaylistManager = value;
      }
    }

    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public BagListManager BagListManager
    {
      get { return _bagListManager; }
      set
      {
        _bagListManager = value;
        libraryControl.BagListManager = value;
        bagListControl.PlaylistManager = value;
      }
    }

    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public SettingsManager SettingsManager
    {
      get { return _settingsManager; }
      set { _settingsManager = value; }
    }

    private void playlistControl_AlbumSelected(Album album)
    {
      libraryControl.SelectAlbum(album);
    }

    public void HandleKey(KeyEventArgs e)
    {
      if (e.KeyCode == Keys.F3)
        libraryControl.FocusSearchBox();
      else if (e.KeyCode == Keys.F6)
        playlistControl.Focus();
      else if (e.KeyCode == Keys.F7 && BagVisible)
        bagListControl.Focus();
      else if (e.KeyCode == Keys.F5)
        libraryControl.ClearRefresh();
      else if (e.KeyCode == Keys.Tab && e.Modifiers.HasFlag(Keys.Control))
      {
        FocusNextControl(backwards: e.Modifiers.HasFlag(Keys.Shift));
      }
    }

    private void FocusNextControl(bool backwards = false)
    {
      if (UIUtils.HasFocus(libraryControl))
        playlistControl.Focus();
      else if (UIUtils.HasFocus(bagListControl) || !BagVisible)
        libraryControl.Focus();
      else
        bagListControl.Focus();

      if (BagVisible && backwards)
        FocusNextControl();
    }

    public void PlaylistSearch(bool bag = false)
    {
      if (bag)
        BagVisible = true;

      using (var playlistSearchToolWin = new PlaylistSearchToolWin())
      {
        playlistSearchToolWin.PlaylistSearched += playlistSearchToolWin_PlaylistSearched;
        playlistSearchToolWin.IsBagSearch = bag;
        playlistSearchToolWin.CanBagSearch = BagVisible;
        playlistSearchToolWin.ShowDialog(ParentForm);

        (playlistSearchToolWin.IsBagSearch ? bagListControl : playlistControl).Focus();
      }
    }

    public bool BagVisible
    {
      get
      {
        return !playlistBagSplit.Panel2Collapsed;
      }
      set
      {
        playlistBagSplit.Panel2Collapsed = !value;
      }
    }

    public bool ToggleBag()
    {
      BagVisible = !BagVisible;
      return BagVisible;
    }

    private List<int?> playlistSearchToolWin_PlaylistSearched(string search)
    {
      List<int?> results = new List<int?>(2);
      results.Add(playlistControl.SearchPlaylist(search));
      results.Add(bagListControl.SearchPlaylist(search));
      return results;
    }
  }
}
