﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Linq;
using Airplayne.Model.Repository;
using System.Drawing.Drawing2D;

namespace Airplayne.Model.Music
{
  public static class CoverHelper
  {
    private static string[] names = new string[] { "front", "cover", "folder" };
    private static List<string> extensions = new List<string> { ".png", ".jpg", ".gif", ".bmp" };

    public static Image GetCover(Album album, int maxDimension)
    {
      string coverPath = GetCoverPath(album);

      if (coverPath == null)
        return null;

      using (Image coverImage = Image.FromFile(coverPath))
      {
        int imageMax = Math.Max(coverImage.Width, coverImage.Height);
        double factor = (double)maxDimension / imageMax;

        int height = (int)(coverImage.Height * factor);
        int width = (int)(coverImage.Width * factor);

        Bitmap thumbnail = new Bitmap(width, height);
        using (var graphics = Graphics.FromImage(thumbnail))
        {
          graphics.SmoothingMode = SmoothingMode.AntiAlias;
          graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;
          graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
          graphics.DrawImage(coverImage, new Rectangle(0, 0, width, height));
        }
        return thumbnail;
      }
    }

    public static bool HasCover(Album album)
    {
      return (GetCoverPath(album) != null);
    }

    public static string GetCoverPath(Album album)
    {
      if (album == null)
        return null;

      string[] files = Directory.GetFiles(album.FolderPath, "*.*", SearchOption.TopDirectoryOnly);

      FileInfo bestCover = GetBestCover(files);

      if (bestCover == null)
        return null;
      else
        return bestCover.FullName;
    }

    private static FileInfo GetBestCover(string[] files)
    {
      return
        files.Where(path => extensions.Contains(Path.GetExtension(path).ToLower()))
        .Select(path => new FileInfo(path))
        .OrderByDescending(coverFile => Evaluate(coverFile))
        .ThenByDescending(coverFile => coverFile.Length)
        .FirstOrDefault();
    }

    private static int Evaluate(FileInfo x)
    {
      int score = names.Length;
      foreach (string name in names)
      {
        if (x.Name.ToLower().Contains(name))
          return score;
        score--;
      }
      return 0;
    }
  }
}
