using System;
using System.Collections.Generic;
using Airplayne.Tools;

namespace Airplayne.Model.Music
{
  public class Track : IComparable, IMusic
  {
    public Track(string fileName)
    {
      this.FileName = fileName;
      this.Title = fileName.Substring(0, fileName.LastIndexOf('.'));
      this.SearchTitle = StringUtils.CleanSearchString(Title, true);
      this.SearchWords = StringUtils.GetWords(SearchTitle);
    }

    public bool Matches(IEnumerable<string> terms)
    {
      foreach (string term in terms)
      {
        if (!SearchTitle.Contains(term))
          return false;
      }
      return true;
    }

    public bool MatchesInAlbum(IEnumerable<string> terms)
    {
      foreach (string term in terms)
      {
        if (!SearchTitle.Contains(term) && !AlbumInstance.SearchTitle.Contains(term))
          return false;
      }
      return true;
    }

    #region Properties

    public Album AlbumInstance
    {
      get;
      set;
    }

    public int? TrackId { get; set; }

    public string FileName { get; set; }

    public string FullPath
    {
      get
      {
        return this.AlbumInstance.FolderPath +
          System.IO.Path.DirectorySeparatorChar +
          this.FileName;
      }
    }

    public string Title
    {
      get;
      private set;
    }

    public string SearchTitle
    {
      get;
      private set;
    }

    public List<string> SearchWords
    {
      get;
      private set;
    }

    #endregion

    #region Equals & friends

    public override string ToString()
    {
      return Title;
    }

    public int CompareTo(object obj)
    {
      Track other = obj as Track;
      if (other == null)
        return 0;

      if (this.AlbumInstance == null || other.AlbumInstance == null)
        return this.FileName.CompareTo(other.FileName);
      else
        return this.FullPath.CompareTo(other.FullPath);
    }

    public override bool Equals(object obj)
    {
      return Equals(obj, false);
    }

    public bool Equals(object obj, bool checkIds)
    {
      Track other = obj as Track;
      if (other == null)
        return false;

      return this.FileName.Equals(other.FileName) && this.AlbumInstance.Equals(other.AlbumInstance)
        && (!checkIds || this.TrackId == other.TrackId);
    }

    public override int GetHashCode()
    {
      return this.FullPath.GetHashCode();
    }

    #endregion

    #region IMusic Members

    public List<Track> Tracks
    {
      get
      {
        return new List<Track> { this };
      }
    }

    #endregion
  }
}
