﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Airplayne.Model.Music;

namespace Airplayne.Model.Repository
{
  public class Library
  {
    public Library()
    {
      Albums = new List<Album>();
      Tracks = new List<Track>();
      TracksByPath = new Dictionary<string, Track>();
      TracksById = new Dictionary<int, Track>();
      AlbumsById = new Dictionary<int, Album>();
    }

    public Library(List<Album> albums)
    {
      Albums = new List<Album>(albums);
      Tracks = LibraryUtils.ExtractTracks(albums);
      SetIds();
      BuildDictionaries();
    }

    private void SetIds()
    {
      for (int i = 0; i < Albums.Count; i++)
        Albums[i].AlbumId = i + 1;
      for (int i = 0; i < Tracks.Count; i++)
        Tracks[i].TrackId = i + 1;
    }

    private void BuildDictionaries()
    {
      AlbumsById = new Dictionary<int, Album>(Albums.Count);
      foreach (Album album in Albums)
      {
        AlbumsById[album.AlbumId.Value] = album;
      }

      TracksById = new Dictionary<int, Track>(Tracks.Count);
      TracksByPath = new Dictionary<string, Track>(Tracks.Count);
      foreach (Track track in Tracks)
      {
        TracksById[track.TrackId.Value] = track;
        TracksByPath[track.FullPath.ToLower()] = track;
      }
    }

    public List<Album> Albums { get; set; }
    public List<Track> Tracks { get; set; }
    public Dictionary<string, Track> TracksByPath { get; set; }
    public Dictionary<int, Track> TracksById { get; set; }
    public Dictionary<int, Album> AlbumsById { get; set; }
  }
}
