﻿using System;
using System.Collections.Generic;
using System.IO;
using Airplayne.Tools;
using System.IO.Compression;
using Airplayne.Model.Repository;
using Airplayne.Model.Music;

namespace Airplayne.Model.Serialization
{
  public class LibrarySerializer
  {
    private static string CACHE_PATH = DeployUtils.GetDataDirectory()
      + Path.DirectorySeparatorChar + "airplayne_librarycache_v4";

    public delegate void LoadProgressedHandler(int percent);
    public static event LoadProgressedHandler LoadProgressed;

    public static void Save(Library library)
    {
      try
      {
        SerializeLibrary(library);
      }
      catch (Exception ex)
      {
        Logger.Write(ex, "Error saving library cache");
      }
    }

    public static Library Load(LoadProgressedHandler progressCallback = null)
    {
      LoadProgressed += progressCallback;
      try
      {
        return DeserializeLibrary();
      }
      catch (Exception ex)
      {
        Logger.Write(ex, "Error loading library cache");
        Clear();
        return null;
      }
      finally
      {
        LoadProgressed -= progressCallback;
      }
    }

    private static void Clear()
    {
      try
      {
        File.Delete(CACHE_PATH);
      }
      catch (Exception ex)
      {
        Logger.Write(ex, "Error clearing library cache");
      }
    }

    private static void SerializeLibrary(Library library)
    {
      using (var writer = new StreamWriter(CACHE_PATH))
      {
        writer.WriteLine(library.Albums.Count);
        foreach (Album album in library.Albums)
        {
          writer.WriteLine(album.FolderPath);
          writer.WriteLine(album.Title);
          foreach (Track track in album.Tracks)
            writer.WriteLine(track.FileName);
          writer.WriteLine();
        }
      }
    }

    private static Library DeserializeLibrary()
    {
      using (StreamReader reader = new StreamReader(CACHE_PATH))
      {
        int albumCount = int.Parse(reader.ReadLine());
        var albums = new List<Album>(albumCount);
        while (!reader.EndOfStream)
        {
          string albumFolder = reader.ReadLine();
          string albumTitle = reader.ReadLine();
          var tracks = new List<Track>();
          for (
            string trackFile = reader.ReadLine();
            !string.IsNullOrEmpty(trackFile);
            trackFile = reader.ReadLine())
          {
            tracks.Add(new Track(trackFile));
          }
          albums.Add(new Album(albumFolder, albumTitle, tracks));
          DoLoadProgressed(albums.Count, albumCount);
        }
        return new Library(albums);
      }
    }

    private static void DoLoadProgressed(int done, int total)
    {
      if (total > 0 && (done == total || done % 10 == 0) && LoadProgressed != null) {
        int percent = done * 100 / total;
        LoadProgressed(percent);
      }
    }
  }
}