﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Airplayne.Model;
using System.IO;
using Airplayne.Model.Music;
using Airplayne.Model.Repository;

namespace Airplayne.Model.Serialization
{
  public class PlaylistSerializer
  {
    private LibraryManager _libraryManager;
    private Encoding _encoding = Encoding.GetEncoding(1252);

    public PlaylistSerializer(LibraryManager libraryManager)
    {
      _libraryManager = libraryManager;
    }

    public void SavePlaylist(Playlist playlist, string path)
    {
      StringBuilder sb = new StringBuilder();
      sb.AppendFormat("#Airplayne Header-{0}-{1}-{2}",
                      playlist.CurrentIndex, playlist.AutopilotIndex, (int)playlist.AutopilotSetting)
        .AppendLine();
      foreach (Track track in playlist.Tracks)
      {
        sb.AppendLine(track.FullPath);
      }
      File.WriteAllText(path, sb.ToString(), _encoding);
    }

    public Playlist LoadPlaylist(string path)
    {
      Playlist playlist = new Playlist();

      string playlistFolder = Path.GetDirectoryName(path);
      string[] lines = File.ReadAllLines(path, _encoding);

      if (lines.Length == 0)
        return playlist;

      SetPropertiesFromAirplayneTag(playlist, lines[0]);
      SetTracksFromLines(playlist, playlistFolder, lines);

      return playlist;
    }

    private void SetTracksFromLines(Playlist playlist, string playlistFolder, string[] lines)
    {
      foreach (string line in lines)
      {
        if (line.StartsWith("#") || string.IsNullOrEmpty(line)) // # = comment or tag info, we don't care either way.
          continue;

        string absoluteTrackPath = line;
        if (!Path.IsPathRooted(absoluteTrackPath))
          absoluteTrackPath = Path.Combine(playlistFolder, line);

        playlist.Tracks.Add(_libraryManager.GetTrack(absoluteTrackPath));
      }
    }

    private void SetPropertiesFromAirplayneTag(Playlist playlist, string tag)
    {
      if (tag.StartsWith("#Airplayne Header"))
      {
        int? currentIndex, autopilotIndex;
        Playlist.AutopilotMode? autopilotSetting;
        ParseAirplayneTag(tag, out currentIndex, out autopilotIndex, out autopilotSetting);
        playlist.AutopilotIndex = autopilotIndex;
        playlist.AutopilotSetting = autopilotSetting;
        playlist.CurrentIndex = currentIndex;
      }
    }

    private void ParseAirplayneTag(string tag, out int? currentIndex, out int? autopilotIndex,
      out Playlist.AutopilotMode? autopilotSetting)
    {
      currentIndex = autopilotIndex = null;
      autopilotSetting = null;

      string[] parts = tag.Split('-');
      if (parts.Length < 4)
        return;

      int currentIndexParsed, autopilotIndexParsed, autopilotSettingParsed;
      if (int.TryParse(parts[1], out currentIndexParsed))
        currentIndex = currentIndexParsed;
      if (int.TryParse(parts[2], out autopilotIndexParsed))
        autopilotIndex = autopilotIndexParsed;
      if (int.TryParse(parts[3], out autopilotSettingParsed))
      try
      {
        autopilotSetting = 
          (Playlist.AutopilotMode)Enum.ToObject(typeof(Playlist.AutopilotMode), autopilotSettingParsed);
      }
      catch (ArgumentException) {}
    }
  }
}
