using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Airplayne.Interface;
using Airplayne.Playback;
using Airplayne.Model;
using Airplayne.Interface.Dialogs;
using System.Threading;
using Airplayne.Tools;
using Airplayne.Model.Repository;
using System.Diagnostics;
using Airplayne.Tools.Threading;

namespace Airplayne
{
  public static class Program
  {
    public delegate void ProgramEvent();
    public static event ProgramEvent Shutdown;

    public static void OnShutdown()
    {
      var handler = Shutdown;
      if (handler != null)
        handler();
    }

    [STAThread]
    static void Main()
    {
      RegisterExceptionHandling();

      WinFormsInit();

      var factory = new Factory();
      factory.Initialize();

      StartUI(factory);
    }

    private static void WinFormsInit()
    {
      Application.EnableVisualStyles();
      Application.SetCompatibleTextRenderingDefault(false);
    }

    private static void StartUI(Factory factory)
    {
      using (MainForm mainForm = factory.GetUI())
      {
        Application.Run(mainForm);
      }
    }

    #region Error handling

    private static void RegisterExceptionHandling()
    {
      AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
      Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);
    }

    private static void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
    {
      EmergencyCatch(e.Exception, false);
    }

    private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
    {
      EmergencyCatch(e.ExceptionObject as Exception, e.IsTerminating);
    }

    private static void EmergencyCatch(Exception e, bool close)
    {
      Logger.Write(e, "Exception uncaught, close forced: {0}", close);
      try
      {
        bool userClose = CrashDialog.ShowException(e, close);
        close = close || userClose;
      }
      finally
      {
        if (close)
        {
          try
          {
            Program.OnShutdown();
          }
          catch (Exception) { }
          Environment.Exit(1);
        }
      }
    }

    #endregion
  }
}