﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Reflection;

namespace Airplayne.Tools
{
  public static class Logger
  {
    private const int LOGSIZE = 102400; // 100 KB. Truncate then add, so it'll be a bit over.

    private static string LogPath
    {
      get
      {
        return DeployUtils.GetDataDirectory() + Path.DirectorySeparatorChar + "log.txt";
      }
    }

    private static void WriteString(string str)
    {
      try
      {
        StackTrace stackTrace = new StackTrace();
        MethodBase method = stackTrace.GetFrames()[2].GetMethod(); // { 0 = WriteString, 1 = Write, 2 = relevant method, ... }
        string methodDescription = string.Format("{0}.{1}", method.DeclaringType.Name, method.Name);
        string path = LogPath;
        string logContent = GetLogContents();
        string addition = string.Format("[{1} - {3} - {0}]{4}{2}", methodDescription, DateTime.Now.ToString(),
          str, DeployUtils.Version, Environment.NewLine);
        logContent = addition + Environment.NewLine + Environment.NewLine + logContent;

        using (StreamWriter writer = new StreamWriter(path, false))
        {
          writer.Write(logContent);
          writer.Flush();
        }
      }
      catch (Exception) { }
    }

    public static void Write(string format, params object[] args)
    {
      string str = string.Format(format, args);
      WriteString(str);
    }

    public static void Write(Exception ex)
    {
      WriteString(ex.ToString());
    }

    public static void Write(Exception ex, string format, params object[] args)
    {
      string str = string.Format(format, args)
        + "\r\n"
        + ex.ToString();

      WriteString(str);
    }

    public static string GetLogContents()
    {
      if (!File.Exists(LogPath))
        return string.Empty;

      using (StreamReader reader = new StreamReader(LogPath))
      {
        string logContent = reader.ReadToEnd();
        if (logContent.Length > LOGSIZE)
        {
          logContent = logContent.Substring(0, LOGSIZE);
        }
        return logContent;
      }
    }
  }
}
