﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Globalization;

namespace Airplayne.Tools
{
  public static class StringUtils
  {

    public static string[] MakeSearchTerms(string search, out string fullSearchString, out string cleanSearch)
    {
      search = CleanSearchString(search, false);
      cleanSearch = search;
      fullSearchString = search.Replace("\"", "");

      // any number of terms, consisting of foo "foo bar" or "foo bar
      Regex splitter = new Regex(@"\""(?<term>[^\""]+)\""?|(?<term>[^\s\""]+)", RegexOptions.Compiled);
      MatchCollection mc = splitter.Matches(cleanSearch);

      List<string> terms = new List<string>();
      foreach (Match m in mc)
      {
        terms.Add(m.Groups["term"].Value);
      }

      return terms.ToArray();
    }

    /// <summary>
    /// Trims, ToLowers and replaces accented characters by their 
    /// non-accented counterparts. Accent magic from:
    /// http://snippets.dzone.com/posts/show/1962 
    /// </summary>
    /// <param name="input"></param>
    /// <returns></returns>
    public static string CleanSearchString(string input, bool removeQuotes)
    {
      input = (input ?? string.Empty).Trim().ToLower();

      string stFormD = input.Normalize(NormalizationForm.FormD);
      StringBuilder sb = new StringBuilder();

      UnicodeCategory uc;
      UnicodeCategory? prev = null;
      for (int i = 0; i < stFormD.Length; i++)
      {
        uc = CharUnicodeInfo.GetUnicodeCategory(stFormD[i]);

        if (uc == UnicodeCategory.LowercaseLetter ||
          uc == UnicodeCategory.DecimalDigitNumber ||
          (uc == UnicodeCategory.SpaceSeparator && prev != uc) || // no double spaces
          (!removeQuotes && stFormD[i] == '"'))
        {
          sb.Append(stFormD[i]);
          prev = uc;
        }
      }

      return sb.ToString();
    }

    public static List<string> GetWords(string cleanString)
    {
      string[] words = cleanString.Split(' ');
      List<string> wordList = new List<string>();
      foreach (string word in words)
        if (word != string.Empty && !wordList.Contains(word))
          wordList.Add(word);

      return wordList;
    }


  }
}
